PRO FOCIS_DISPLAY, input, SAVEFILE=savefile, INFILE=infile, DISPLAY_SIZE=display_size, OVERVIEW_PLOTS=overview_plots, PS=ps, DYN_SPECTRA=dyn_spectra, CHANNELS=channels
;
; Procedure to display FOCIS output.
;
;  2-24-06      Initial version, adapted from inline code in FOCIS.  (ghurford@ssl.berkeley.edu)
;  2-25-06 gh   Change relative time units to milliseconds.
;               Add INFILE keyword to support input via file restoration.
;  5-11-06 gh   Adapt to change in input timetag units from ms to seconds.
;  22-May-06 gh Change calls to fst_overview_plots and fst_spectrogram_display to support display of absolute time.
;  26-May-06 gh Correct recently introduced bug associated with use of direct input.
;
;
; INPUT:
;   Input is a focisout structure containing the data to be displayed.
;   If input is not specified but SAVEFILE is, then the input data is restored from SAVEFILE.
;   If neither input nor savefile is specified, then the input data is read from an input file created by fst_focisout_reformat.
;   If INFILE is not specified, the default infile name is 'focisout.dat'
;
;
; Get data, if necessary.
DEFAULT, infile, 'focisout.dat'
IF N_ELEMENTS(input) EQ 0 THEN BEGIN
    IF N_ELEMENTS(savefile) EQ 1 THEN RESTORE, savefile, /V     $
    ELSE focisout =  fst_focisout_read(lunin, FILENAME=infile, /NEWFILE)
    IF N_ELEMENTS(focisout) EQ 0 THEN MESSAGE, 'Nonstandard savefile.  Suggest restoring ', infile, ' manually.'
    input = focisout
ENDIF
;
; Regardless of the input option, data are now in a structure, 'input'.
; Reorganize time units to be compatible with UTPLOT. t0 is the time of 1st datum in ANYTIM format. reltime is seconds relative to t0.
t0 = ANYTIM(STRMID(input.acqheader.scanid,0,4)+'/'+STRMID(input.acqheader.scanid,4,2)+'/'+STRMID(input.acqheader.scanid,6,2)) + input.timesec[0]
reltime = input.timesec - input.timesec[0]
PRINT, ANYTIM(T0,/ECS)
;
; Set defaults and initialize.
DEFAULT, display_size,      [0.625, 0.625]
DEFAULT, ps,                0
DEFAULT, nods,              1
DEFAULT, overview_plots,    1
DEFAULT, dyn_spectra,       0
DEFAULT, channels,          INDGEN(input.acqheader.nchan)+1		; channel numbers to be processed (=1,2,3 and/or 4)
nch     = N_ELEMENTS(channels)					; number of channels to be processed
ich	    = channels -1							; vector of indices for channels to be processed
nap	    = nch*(nch-1)/2							; number of antenna pairs
iap	    = [[0,1], [1,2], [2,0]]					; indices for antenna pairs.
;
; Overview plots
IF overview_plots NE 0 THEN BEGIN
	IF ps NE 0 THEN BEGIN
		SET_PLOT,  'ps'
		DEVICE, /COLOR
	ENDIF
	TITLE=[	'Dynamic Power Spectra Histograms', $
			'Frequency-averaged Light Curves',  $
			'Time-averaged Spectra']
	fst_overview_plots, input.pwrspect, reltime, input.freqmhz, t0, TITLE=title,						$
						PS = ps, CHLABELS=input.acqheader.chandesc[channels-1], COLORS=[255,160, 80]
	IF nap GT 0 THEN BEGIN
	    TITLE=[ 'Dynamic Correlated Power Histograms', 			$
		    	'Frequency-averaged Correlated Light Curves', 	$
			    'Time-averaged Correlated Spectra']
	    fst_overview_plots, ABS(input.corspect), reltime, input.freqmhz, t0, TITLE=title, 		$
 		    				PS = ps, COLORS=[255,254,128], 					$
 			    			CHLABELS=input.acqheader.chandesc[channels[iap[0,*]]-1]+' x '+ input.acqheader.chandesc[channels[iap[1,*]]-1]
    ENDIF
	IF ps NE 0 THEN BEGIN
		DEVICE, /CLOSE
		set_plot, 'win'
	ENDIF
ENDIF
;

plotman_obj=plotman(colortable=39,/multi)
; Loop over channels and antenna pairs for spectrogram displays
IF dyn_spectra NE 0 THEN BEGIN
	FOR nc = 0, nch-1 DO BEGIN
		ic = channels[nc]-1
		fst_spectrogram_display, input.pwrspect[*,*, nc], input.timesec, input.freqmhz, t0, $
				TITLE='Total Power   '+input.acqheader.chandesc[ic], DISPLAY_SIZE=display_size, $
				plotman_obj=plotman_obj;, paneldesc='Total Power   '+input.acqheader.chandesc[ic]
	ENDFOR
	IF nap GT 0 THEN BEGIN
		FOR nb = 0, nap-1 DO BEGIN
			jc = iap[*, nb]
			fst_spectrogram_display, ABS(input.corspect[*,*, nb]), input.timesec, input.freqmhz, t0, $
				TITLE='Correlated Amplitude   '+input.acqheader.chandesc[jc[0]]+' x '+input.acqheader.chandesc[jc[1]],	$
				DISPLAY_SIZE=display_size,plotman_obj=plotman_obj;, paneldesc= 'Correlated Amplitude   '+input.acqheader.chandesc[jc[0]]+' x '+input.acqheader.chandesc[jc[1]]
			phase = (ATAN(input.corspect[*,*,nb], /PHASE)*!RADEG +540.) MOD 360. -180.
			fst_spectrogram_display, phase, input.timesec, input.freqmhz, t0, $
				TITLE='Correlated Phase   '+input.acqheader.chandesc[jc[0]]+' x '+input.acqheader.chandesc[jc[1]],	$
				DISPLAY_SIZE=display_size, /LINEAR,plotman_obj=plotman_obj ;, paneldesc='Correlated Phase   '+input.acqheader.chandesc[jc[0]]+' x '+input.acqheader.chandesc[jc[1]]
		ENDFOR
	ENDIF
ENDIF
RETURN
END
