;+
; NAME:
;       fst_idl2lv
;
; PURPOSE:
;       This procedure saves and IDL structure as a LabVIEW variant record
;
; CATEGORY:
;       Data writers
;
; CALLING SEQUENCE:
;        fst_idl2lv,data,file,attributes=attributtes
;
; INPUTS:
;       FILE:   The datalog file name. If not provided, a generic 'variant.dat' is created.
;
; OUTPUTS:
;       None
; KEYWORD PARAMETERS:
;       ATTRIBUTES an IDL structure containing the attributes to be added to the LabVIEW variant
; REFERENCE:
;       National Instruments Application Note 154
;
; RESTRICTIONS:
;       Keyword ATTRIBUTES must be an IDL structure, otherwise it is ignored
;       This program was tested using LabVIEW 7.1
;       Its compatibility with older LabView versions cannot be guaranteed.

; DISCLAIMER:
;       Use this program at your own risk.
;
; MODIFICATION HISTORY:
;       Written by:  Gelu M. Nita (gnita@njit.edu), Jan 7, 2006
;       07-March-2006 Making use of forward_function to declare get_variant
;-
;***********************************************
;IDL type codes are translated here to corresponding LabVIEW type
;***********************************************
function idl2lv_code, data
    Case size(data,/tname) of
	    "BYTE":    code=5

		"INT":     code=2

		"LONG":    code=3

		"FLOAT":   code=9

		"DOUBLE":  code=10

		"COMPLEX": code=12

		"STRING":  code=48

		"STRUCT":  code=80

		"DCOMPLEX":code=13

		"UINT":    code=6

		"ULONG":   code=7

        "POINTER": code=83

         else:     code=0
     endcase
     return,code
end
;***********************************************
function empty_variant
 return,{record_length:12L,type_length:4u,type_code:0u,n_attributes:0l}
end
;***********************************************
forward_function get_variant
;***********************************************

function lv_encode,data,set_name=set_name,clear=clear
 if n_elements(data) eq 0 then return,{typestr:0,datastr:0l}; Undefined data, nothing to encode
 type=size(data,/tname)
 n_dim=size(data,/n_dim)
 IF n_dim EQ 0 THEN BEGIN
 ;This is a scalar
	 case type of
	 "POINTER":begin
               typestr={type_length:0u,type_code:idl2lv_code(data)}
               if ptr_valid(data) then datastr=get_variant(*data) else return,empty_variant();datastr=empty_variant()
	           end
	 "STRING":begin
			  typestr={type_length:0u,type_code:idl2lv_code(data),fake_length:-1l}
			  datastr={real_length:strlen(data),datastr:byte(data)}
			  end
	  else: begin
			  ;any other scalar type
			  typestr={type_length:0u,type_code:idl2lv_code(data)}
			  datastr={datastr:data}
	         end
	 endcase
 ENDIF ELSE BEGIN
 ;This is an array,a structure,or an array of pointers
  case type of
   "POINTER":BEGIN
              ;This is an array of pointers
		       for k=0, n_elements(data)-1 do begin
		        if ptr_valid(data[k]) then element=get_variant(*data[k]) else element=empty_variant()
		        case n_elements(elements) of
		         0:elements=Create_Struct(strcompress(string(k,format="('Variant',i10)")),element)
		         else:elements=Create_Struct(elements,Create_Struct(strcompress(string(k,format="('Variant',i10)")),element))
		        endcase
		       end
		       descriptor={type_length:0u,type_code:83}
		       descriptor.type_length=n_tags(descriptor,/data)
		       datastr={dimensions:size(data,/dimensions),datarecord:elements}
	           typestr={type_length:0u,array_code:64,n_dim:fix(n_dim),fake_dim:replicate(-1l,n_dim),element_descriptor:descriptor}
             END
   "STRUCT":BEGIN
             n=n_tags(data)
             tags=tag_names(data)
             replicas=n_elements(data)
             IF replicas EQ 1 THEN BEGIN
             ;This is an unreplicated structure
             for i=0l, n-1 do begin
              code=lv_encode(data.(i),set_name=tags[i])
              if (n_elements(typestr) eq 0) then begin
                 typestr=Create_Struct(tags[i],code.typestr)
                 datastr=Create_Struct(tags[i],code.datastr)
	          endif else begin
	                 typestr=Create_Struct(typestr,tags[i],code.typestr)
	                 datastr=Create_Struct(datastr,tags[i],code.datastr)
	          endelse
             endfor
             typestr=Create_Struct("type_length",0u,"type_code",idl2lv_code(data),"n_elements",fix(n),"typestr",typestr)
             ENDIF ELSE BEGIN
              ;This is a replicated structure
              for k=0l, replicas-1 do begin
                code=lv_encode(data[k])
		        case n_elements(elements) of
		         0:elements=Create_Struct(strcompress(string(k,format="('Element',i10)")),code.datastr)
		         else:elements=Create_Struct(elements,Create_Struct(strcompress(string(k,format="('Element',i10)")),code.datastr))
		        endcase
		      end
		      descriptor=code.typestr
		      descriptor.type_length=n_tags(descriptor,/data)
		      datastr={dimensions:replicas,datarecord:elements}
	          typestr={type_length:0u,array_code:64,n_dim:fix(n_dim),fake_dim:replicate(-1l,n_dim),element_descriptor:descriptor}
             ENDELSE
             END
   else: BEGIN
          ;This is an array of strings or scalars
	      if type eq "STRING" then begin
	       ;This is an array of strings
	       for k=0, n_elements(data)-1 do begin
	        case n_elements(elements) of
	         0:elements=Create_Struct(strcompress(string(k,format="('length',i10)")),strlen(data[k]),strcompress(string(k,format="('string',i10)")),byte(data[k]))
	         else:elements=Create_Struct(elements,Create_Struct(strcompress(string(k,format="('length',i10)")),strlen(data[k]),strcompress(string(k,format="('string',i10)")),byte(data[k])))
	        endcase
	       end
	       descriptor={type_length:0u,type_code:idl2lv_code(data),fake_length:-1l}
	       descriptor.type_length=n_tags(descriptor,/data)
	      endif else begin
	       ;This is a numerical array
	       elements=data
	       descriptor=[4u,idl2lv_code(data)]
	      end
	      datastr={dimensions:size(data,/dimensions),datarecord:elements}
	      typestr={type_length:0u,array_code:64,n_dim:fix(n_dim),fake_dim:replicate(-1l,n_dim),element_descriptor:descriptor}
         end
         END

 END
 ;Check if the variable has a name and act accordingly
 if size(set_name,/tname) eq "STRING" then nstr=set_name else  nstr=SCOPE_VARNAME(data,level=-1000)

  if nstr ne "" then begin
	 ;Convert string to byte array (Required for later getting the correct structure length)
	 nstr=byte(nstr)
	 chars=byte(n_elements(nstr))
	 ;Pad the variable name to reach a byte boundary (LabView internal requirement)
	 if (n_elements(nstr)+1) mod 2 eq 1 then nstr=[nstr,0b]
	 typestr=Create_Struct(typestr,"nchars",chars,"name",nstr)
  end
  ;Set the typedescriptor length
  typestr.type_length=n_tags(typestr,/data)
  return,{typestr:typestr,datastr:datastr}
end

;***********************************************
function get_variant,data,attributes=attributes
 if (n_elements(data) gt 0) then begin
  code=lv_encode(data)
     if n_tags(code) eq 4 then begin
	  if code.(0) eq 12 and code.(1) eq 4 and code.(2) eq 0 and code.(3) eq 0 then begin
	   nil=1
	  end
	 endif
	 if keyword_set(nil) then begin
	   variant=code
	   variant.n_attributes=n_tags(attributes)
	 endif else begin
	   variant={record_length:0l,typestr:code.typestr,datastr:code.datastr,n_attributes:n_tags(attributes)}
	 endelse

 endif else variant={record_length:12L,type_length:4u,type_code:0u,n_attributes:n_tags(attributes)}
 for i=0,n_tags(attributes)-1 do begin
  tags=tag_names(attributes)
  tag=tags[i]
  key=byte(tag)
  if (strlen(tag)+1) mod 2 eq 1 then key=[key,0b]
  key_length=n_elements(key)
  value=get_variant(attributes.(i))
  variant=Create_Struct(variant,tag+'_length',key_length,tag+"_key",key,tag,value)
 end
 variant.record_length=n_tags(variant,/data)
 return,variant
end
;***********************************************

;***********************************************
pro fst_idl2lv,data,file,attributes=attributes
 variant=get_variant(data,attributes=attributes)
 if n_elements(file) eq 0 then file='variant.dat'
 openw,unit,file,/get_lun,/SWAP_IF_LITTLE_ENDIAN
 writeu,unit,variant
 free_lun,unit
end
;***********************************************
