;+
; NAME:
;     fst_lvdata
; PURPOSE:
;     To read data from a LabView file
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;        fst_lvdata(unit,lvtemplate,[exit_point])
; INPUTS:
;        unit:  IDL logical file unit of the file containing the type descriptor
;        lv_template: data template, as the one returned by labview_template function
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;        exit_point: the file pointer position corresponding to the end of the record being read
; ROUTINES CALLED:lvcode70
; OUTPUTS:IDL structure or numeric data template
; COMMENTS:
; SIDE EFFECTS: If an an unknown type code is found, the returned value contains
;               valid data up to the point where the error occured.
;               If the exit_point keyword is provided, the unknown type code and the rest of the record are returned  as hexadecimal strings
;               in a special UNKNOWN structure.
;
; RESTRICTIONS:
;              Only some LabView data types are supported. More will be added as needed
;              An IDL data template, as the one returned by labview_template function, should be provided.
;              The file pointer of the logical unit provided should be at the beginning of the data section
; MODIFICATION HISTORY:
;     Written 25-Jan-2005 by Gelu Nita
;     Rewriten entirely 07-March-2006 GN
;-

;--------------------------------------------------------------------------

function fst_lvdata,type_unit,data_unit,first_call=first_call,entry_point=entry_point,variable_length=variable_length,no_warning=no_warning

    ;If not defined yet, assume that this is a fixed length record
    ;If a variable length data type (string or variant) is later detected
    ;this keyword is set to 1 to flag this situation.
     if n_elements(variable_length) eq 0  then variable_length=0

    ;This is the entry point. We need it to keep track of the bytes locally read.
     POINT_LUN,-type_unit,entry_point

    ;The first WORD is the type descriptor length in bytes
     type_length=0u
     readu,type_unit,type_length

    if keyword_set(first_call) then begin
     ;Point the data pointer at the begining of data section
     SKIP_LUN,data_unit,type_length
    end

    ;The second WORD is the type code,
    ;but only the least sigificant bit is meaningfull for our purpose
    ;(the most significant bit is reserved by LabView for internal purposes)
     type_code=0u
     readu,type_unit,type_code

    ;Some of the LabView 7.0 codes are defined here
    lvcode=fst_lvcode70()

    ;Check if the current LabView control has any customized type definition
    ;This step is nedeed just to advance the file pointer to
    ;the begining of the control type descriptor
     if (type_code and 255) eq lvcode.typedef then BEGIN
      ;Read the type definition code
      typedef=0l
      readu,type_unit,typedef
      ;Read the type definition name
      length=0b
      readu,type_unit,length
      ctl=bytarr(length)
      readu,type_unit,ctl
      ;Make sure the file pointer is at an even byte boundary
      SKIP_LUN,type_unit,(length+1) mod 2
      ;Read the length of the control type descriptor, even if we don't use it
      ctl_length=0u
      readu,type_unit,ctl_length
      ;Read the control type code that will be further analyzed
      readu,type_unit,type_code
     END

     CASE (type_code and 255) of

      lvcode.variant:BEGIN
                     status=fstat(data_unit)
                     openr,variant_unit,status.name,/get_lun,/SWAP_IF_LITTLE_ENDIAN
                     point_lun,-data_unit,variant_start
                     point_lun,variant_unit,variant_start
                     data=ptr_new(fst_variant(variant_unit,data_unit,/first))
                     ;The variant length is not predefined, so we need to flag this
                     variable_length=1
                     free_lun,variant_unit
                   END

      lvcode.empty:BEGIN
                     ;This is an empty variant data container
                     data={empty_variant,container:ptr_new()}
                     ;The variant length is not predefined, so we need to flag this
                     variable_length=1
                   END

      lvcode.i8:BEGIN
                 ;IDL does not have a signed signed byte integer type, so the user should know it somehow
                if keyword_set(no_warning) eq 0 then i8=1
                 data=0b
                 readu,data_unit,data
                END

      lvcode.u8:BEGIN
                 data=0b
                 readu,data_unit,data
                END

      lvcode.e8:BEGIN
                 data=0b
                 goto, enumerate
                END

      lvcode.i16:BEGIN
                 data=0s
                 readu,data_unit,data
                END

      lvcode.u16:BEGIN
                 data=0u
                 readu,data_unit,data
                END

      lvcode.e16:BEGIN
                 data=0u
                 goto, enumerate
                END

      lvcode.i32:BEGIN
                 data=0l
                 readu,data_unit,data
                END

      lvcode.e32:BEGIN
                 data=0l
                 enumerate:
                 readu,data_unit,data
                 elements=0u
                 readu,type_unit,elements
                 names=strarr(elements)
                 total_length=0
                 for e=0,elements-1 do begin
                  name_length=0b
                  readu,type_unit,name_length
                  total_length=total_length+name_length+1
                  name=bytarr(name_length)
                  readu,type_unit,name
                  names[e]=string(name)
                 end
                 ;Make sure the file pointer is at an even byte boundary
                 SKIP_LUN,type_unit,total_length mod 2
                END

      lvcode.u32:BEGIN
                 data=0ul
                 readu,data_unit,data
                END

      lvcode.single:BEGIN
                 data=float(0)
                 readu,data_unit,data
                END

      lvcode.double: BEGIN
                     data=0d
                     readu,data_unit,data
                    END

      lvcode.complex:BEGIN
                      re=float(0)
                      im=float(0)
                      readu,data_unit,re
                      readu,data_unit,im
                      data=complex(re,im)
                     END

      lvcode.dcomplex:BEGIN
                      re=0d
                      im=0d
                      readu,data_unit,re
                      readu,data_unit,im
                      data=dcomplex(re,im)
                     END
      lvcode.boolean:BEGIN
                      data=0b
                      readu,data_unit,data
                     END

      lvcode.string:BEGIN
                    ;This is a LONG PASCAL string
                    string_length=0L
                    readu,type_unit,string_length
                    readu,data_unit,string_length
                    if string_length gt 0 then begin
                     data=bytarr(string_length)
                     readu,data_unit,data
                     data=string(data)
                    endif else data=''
                    ;The string length may be variable, so we need to flag this
                    variable_length=1
                   END

     lvcode.array:BEGIN
                     ;Read the number of dimensions (word type)
                     dimensions=0u
                     readu,type_unit,dimensions
                     arr_sizes=lonarr(dimensions)
                     ;This is just a blind reading because LabView does not store the real information here
                     readu,type_unit,arr_sizes
                     ;This is the actual reading
                     readu,data_unit,arr_sizes
                     ;Now read the the array elements
                     nelements=product(arr_sizes)
                     if nelements gt 0 then begin
                      for s=0l, nelements-1 do begin
                       ;if (s ne (nelements-1)) then array=1 else array=0
                       if n_elements(data) eq 0 then begin
                        first=fst_lvdata(type_unit,data_unit,entry_point=entry,variable_length=variable_length)
                        data=replicate(first,nelements)
                       endif else begin
                        if variable_length eq 1 then begin
                         ;If we have to read each element one by one, the type_unit pointer should be repositioned each time
                         ;such as to point to the element type descriptor.
                         ;This results in low code performance but it is the only solution I could think of in the case
                         ;of variable length element types such as strings, variants, or structured types containing them.
                         point_lun,type_unit,entry
                         data[s]=fst_lvdata(type_unit,data_unit,entry_point=entry,/no)
                        endif else goto,fixed_length
                       end
                      end
                      if 0 eq 1 then begin
                       ;This point is reached only if the above loop is exited before ending
                       fixed_length:
                        buffer=data[1:*]
                        readu,data_unit,buffer
                        data[1:*]=buffer
                      end
                      if dimensions gt 1 then data=REFORM(data, rotate(arr_sizes,2), /OVERWRITE)
                      endif else begin
		                  typelength=0u
		                  readu,type_unit,typelength
		                  typestr=bytarr(typelength-2)
		                  readu,type_unit,typestr
		                  descriptor=''
		                  for s=0,n_elements(typestr)-1 do descriptor=descriptor+string(typestr[s],format='(Z02)')
		                  data={empty_array,element_descriptor:descriptor};This signals an empty array
                          variable_length=1
                      end
                   END

     lvcode.cluster: BEGIN
                          ;This is a Cluster data type. It can be
                          ;represented as an IDL structure, if the names of the variables are encoded,
                          ;or as an array of identical unnamed elements

                         ;Get number of elements in the structure
                          nelements=0u
                    readu,type_unit,nelements
                    ;Now decode each structure element and add it to the structure
                     for i=0, nelements-1 do begin
                      CASE size(data,/type) of
                       0:data=fst_lvdata(type_unit,data_unit,variable_length=variable_length)
                       8:data=Create_Struct(data,fst_lvdata(type_unit,data_unit,variable_length=variable_length))
                       else:data=[data,fst_lvdata(type_unit,data_unit,first_call=first_call,variable_length=variable_length)]
                      ENDCASE
                     end
                     END

       lvcode.waveform:BEGIN
             ;Waveforms have a subtype for LabView internal use,
             ;we read it just go further.
             subtype_code=0u
             readu,type_unit,subtype_code
             ;Now we can read the type descriptor of the waveform.
             case subtype_code of
             6: begin
             	;If the subtype_code=6 the "waveform" is in fact a TimeStamp.
             	;Don't ask me why NI defined the TimeStamp (128bit integer)
             	;as a waveform subtype
             	;The ulong type cast is added to fix an apparent bug in the LabView
             	;TimeStamp type stamp definition, which it is claimed to be a sequence of
             	;four positive 32 bit integers, while the returned values in IDL are negative numbers.
             	;I assume here that in fact they are unsigned 32 bit integers.
             	data=ulong(fst_lvdata(type_unit,data_unit))
             	end
             else:data=fst_lvdata(type_unit,data_unit)
             endcase
            END
     else:message,"Not supported LabView data type:"+string(type_code,format='(Z)')
    ENDCASE

    ;There might be some extra bytes unread, that are supposed to hold the variable name, if any
       POINT_LUN,-type_unit,count
       name=''
       bytes_left=type_length-(count-entry_point)
       if bytes_left gt 0 then begin
        ;Read the name length
        name_length=0b
        readu,type_unit,name_length
        if name_length gt 0 then begin
         name=bytarr(name_length)
         readu,type_unit,name
         name=string(name)
         ;Make sure the file pointer is at an even byte boundary
         SKIP_LUN,type_unit,(name_length+1) mod 2
        end
       end
       ;If the variable has a name we return it as a structure, otherwise as an unnamed variable
       if name ne '' then data=Create_Struct(name,data)
       if keyword_set(i8) then begin
        print,'WARNING:'+name+' LabView signed byte converted to IDL usigned byte!'
       end
       return,data
 end
