;+
; NAME:
;     FST_MKPARM
; PURPOSE:
;     Creates file for use by the FASR Subsystem Testbed that will contain various
;     parameters needed for the header of FST files.  This is called by Watcher on
;     receipt of a header, in order to help synchronize FST's information with the
;     schedule.
; CATEGORY:
;     OVRO APC REALTIME
; CALLING SEQUENCE:
;     fst_mkparm,filename,hrec
; INPUTS:
;     filename	The name of the input file to read from (usually DAILY.ARC)
;     hrec		The record number of the header from which to obtain relevant FST
;       		  parameters.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
;
; OUTPUTS:
; COMMENTS:
; SIDE EFFECTS:
;     If the scan type is one of SOLAR, SATOBS, or PCAL, the file GEOMETRY.SAV is
;     written to the working directory. Any existing file of that name is overwritten.
; RESTRICTIONS:
; MODIFICATION HISTORY:
;     Written 08-Jan-2006 by Dale E. Gary
;     10-Jan-2006  DG
;       First tentative working version, simply writes out GEOMETRY structure
;     04-Mar-2006  DG
;       Now converts to LabVIEW format, for directly ingesting to LabVIEW.
;-

pro FST_mkparm,filename,hrec

   ;This is the code to handle the errors
   error=''

   CATCH, Error_status

   ;This statement begins the error handler.
   IF Error_status NE 0 THEN BEGIN
      error='FST_mkparm:'+!ERR_STRING
      PRINT, 'Error:', error
      CATCH,/Cancel
      if n_elements(lun) ne 0 then if lun ne 0 then free_lun,lun
      return
   ENDIF

   ; If no filename, assume standard DAILY.ARC file
   IF (n_elements(filename) EQ 0) THEN $
                  filename=!defaults.workdir+'DAILY.ARC'
   ; Open the file
   lun = openarc(filename,a,nrec,auto=auto)
   if lun eq 0 then message, 'OPENARC:cannot open this file'
   ; Read header info, etc.
   rec = newscan(a,nrec,hrec,header,obseq,cfg,traj,refcal,gparm,geometry,pRecent,pOld,cyc_str,pshift=pshift) - 1

   ; Sanity check on NEWSCAN result
   if (rec le 0) then begin
      print,'FST_MKPARM: Error reading scan information from file ',filename,' record ',hrec
      return
   endif

   skip = 0
   ; Check that this is a desired type of scan (one of SOLAR, SATOBS, or PHASE CAL)
   case header.tls.scancode of
      !scan.solar:
      !scan.satobs:
      !scan.pcal:
      else: skip=1
   endcase

   if (skip eq 0) then begin
      fst_idl2lv,geometry,!defaults.workdir+'geometry.lv'
;      save,file=!defaults.workdir+'geometry.sav',geometry
      print,'FST_MKPARM: Successful Completion.'
   endif

   ; Free memory associated with structures
   freescan,obseq,traj,refcal,gparm,geometry,pRecent,pOld,cyc_str

return
end