;+
; NAME
;    FST_PCOR
;
; DESCRIPTION
;    Return FST phase correction for a processed FST data block
;
; CALLING SEQUENCE
;    pcor = FST_PCOR(nap,chanid,taug,RF,LO,bandswitch,sidebandswitch)
;
; INPUTS
;    nap			Number of amplitude and phase channels (correlations)
;    chanid         Array of channel IDs from ACQHEADER (ACQHEADER.CHANID)
;    taug           Array of geometrical delays returned by DELAY for this data block,
;                     of size [7,7].
;    RF             Array of RF frequencies, in MHz (size depends on block size)
;    LO             Value of local oscillator frequency in MHz (ACQHEADER.LOMHz)
;    bandswitch     Value of band switch setting (ACQHEADER.BAND)
;    sidebandswitch Value of sideband switch setting (ACQHEADER.SIDEBAND)
;
; Modification History
;    Written 04-Jul-2006 by Dale E. Gary
;-

function fst_pcor,nap,chanid,taug,RF,LO,bandswitch,sidebandswitch

   ; bandswitch = 1, 0 for 1-5 GHz, 5-9 GHz, but band = 0, 1 for 1-5 GHz, 5-9 GHz
   ; sidebandswtich = 1, 0 for lower, upper, but sigma = -1,1 for lower, upper
   band = 1-bandswitch
   sigma = sidebandswitch ? -1 : 1

   nf = n_elements(RF)
   pcor = complexarr(nf,nap)
   tau = dblarr(nap)

   FOR bl=0,nap-1 DO BEGIN

	 IF nap EQ 1 THEN BEGIN
        ; ChanID = 1, 2, or 3 for ants 5, 6, or 7, so adding chanids gives unique total 3, 4, or 5
        ; BLID is then the baseline ID, 0, 1 or 2
	    if (chanid[0] eq 1 and chanid[1] eq 2) then blid = 0
	    if (chanid[0] eq 2 and chanid[1] eq 3) then blid = 1
  	    if (chanid[0] eq 1 and chanid[1] eq 3) then blid = 2
	 ENDIF ELSE blid = bl

	 CASE blid OF
	    0:tau[bl]=taug[3,4]        ; Baseline 5 x 6
	    1:tau[bl]=-taug[4,5]       ; Baseline 6 x 7
	    2:tau[bl]=taug[3,5]        ; Baseline 7 x 5
 	 ENDCASE
  ENDFOR

   ; Code below implements phase correction according to expression
   ;       phi = (sigma*w_BL - sigma*w_LO - w_D)*tau +/- sigma*w_RF*tau_0
   ; where the + sign is for lower band switch setting, and - sign for upper band.
   ; The last term is nominally a constant, and hence ignorable, except that delay tau is
   ; quantized to the nearest ns (tau1), hence the non-integer part of the delay,
   ; delta = tau1 - tau must be corrected.  Actual expression, then, is:
   ;       phi = (sigma*w_BL - sigma*w_LO - w_D)*tau1 +/- sigma*w_RF*delta
   ; where w_RF is angular frequency of RF.  Frequencies are expressed in GHz
   ; because tau is in ns.

   w_LO = 2*!pi*LO/1000.   ; Angular frequency of Local Oscillator
   w_D  = 2*!pi*1.0d       ; Angular frequency of digitizer downconversion
   w_RF = 2*!pi*RF/1000.   ; Angular frequency of RF (array of size NF)
   IF band EQ 1 THEN BEGIN
      ; Upper band
      w_BL = 2*!pi*10.0d   ; Angular frequency of block downconversion (10 GHz)
      sgn = -1
   ENDIF ELSE BEGIN
      ; Lower band (block downconversion frequency is zero)
      w_BL = 0.0d
      sgn = +1
   ENDELSE

   ; Determine delay error due to application of integer delay (in nsec) in FOCIS
   tau1=ROUND(tau)
   if (nap eq 3) then tau1[0]=-(tau1[1]+tau1[2])  ; NB: different than ROUND(tau[*,1]+tau[*,2])
   ; Non-integer part of delay
   delta=tau1-tau

   FOR bl=0,nap-1 DO BEGIN
      phase_cor=reform((sigma*(w_BL - w_LO) - w_D)*tau1[bl]#replicate(1d,nf) $
   					+ sgn*sigma*delta[bl]#w_RF)
   	  pcor[*,bl] = COMPLEX(cos(-phase_cor),sin(-phase_cor))
   ENDFOR


return,pcor

END
