PRO FST_PROCESS_ANALYSIS_BLOCK, block, chanid, power_out, correl_out, psys
;
; Converts one block of FST multi-channel data into dynamic power spectra and visibility spectra.
;
; INPUT: 	block is a 2-dimensional (nsamp x nch) FP array of samples.
;           PSYS an array (one for each antenna) giving the power (dbm, integrated over the band) for each antenna.
;
; OUTPUT:	power_out is the 2-dimensional FP power array (nf x nch), normalized to reproduce psys.
;			If nch >1, correl_out is the 2-dimensional complex visibility array (nf x nap)
;			nf = nsamp/2			; note that this includes the 'zero frequency'
;			nap = nch*(nch-1)/2
;
; 16-Aug-05 gh		Initial version, adapted from single channel fst_process_block. (ghurford@ssl.berkeley.edu)
; 26-Feb-06 gh      Add provision for psys.
;                   Debug 1-baseline case
; 25-May-06 gh      Suppress renormalization if psys values are set to -1.
; 17-Jun-06 dg      Fix bug in case of one baseline consisting of ants 5 x 7, need to invert order of
;                   correlation.  Added CHANID input.
;
blockdim	= SIZE(block, /DIMENSIONS)
iap			= [[0,1], [1,2], [2,0]]				; indices for antenna pairs.
nsamp		= blockdim[0]
nch			= blockdim[1]
nap 		= nch*(nch-1)/2
nf 			= nsamp / 2
fftb 		= FFT(block, DIMENSION = 1)
power_out 	= REFORM((ABS(fftb[0:(nf-1),*]))^2, nf, nch)
factor      = FLTARR(nch)
;
; Special case: if baseline is 5 x 7, then need to invert order of correlation.                    ; DG/gh
IF nap EQ 1 THEN IF chanid[0] EQ 1 AND chanid[1] EQ 3 THEN iap = [[1,0],[1,2],[2,0]]               ; DG/gh
;
; Normalize to psys, if it is not set to its flagged value of -1.
FOR n=0, nch-1 DO BEGIN
    IF psys[n] NE -1 THEN BEGIN
        integrated_power = TOTAL(power_out[1:*,n], 1)       ; scalar
        factor[n]        = psys[n] / integrated_power
        power_out[*,n]   = power_out[*,n] * factor[n]
    ENDIF
ENDFOR
IF nch EQ 1 THEN RETURN
;
fftbj		= CONJ(fftb[0:(nf-1),*])
correl_out	= REFORM(COMPLEXARR(nf, nap), nf, nap)			; forced to be a 2-D array, even if nap=1
FOR n = 0, nap-1 DO BEGIN
	i1 				= iap[0,n]
	i2 				= iap[1,n]
    correl_out[*,n]	= fftb[0:(nf-1),i1] * fftbj[*, i2]		; complex (nf x nap) array of visibilities
    IF psys[i1] NE -1 AND psys[i2] NE -1 THEN BEGIN
        factorn         = SQRT(factor[i1]*factor[i2])
        correl_out[*,n] = correl_out[*,n] * factorn             ; normalization now matches total power
    ENDIF
ENDFOR
RETURN
END