;+
; NAME:
;     fst_startfreq
; PURPOSE:
;     Returns the RF start frequency of the frequency band
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;     header=fst_startfreq(datin[,basefreq=basefreq][,ifsense=ifsense][,band_id=band_id])
; INPUTS:
;     datin: one FST data record
;
; OPTIONAL (KEYWORD) OUTPUT PARAMETERS:
;     basefreq: RF frequency for IFfreq=0
;     ifsense : 1,-1 if RF and IF frequencies have same,different sense
;     band_id : index for equivalent "standard" band, given as follows
;               Freq. Range   Index    Freq. Range   Index
;                1000-1500		0		5000-5500	   10
;				 1500-2000		1		5500-6000	   11
;				 2000-2500		2		6000-6500	   12
;				 2500-3000(U)	3		6500-7000(U)   13
;				 2500-3000(L)	4		6500-7000(L)   14
;				 3000-3500(U)	5		7000-7500(U)   15
;				 3000-3500(L)	6		7000-7500(L)   16
;				 3500-4000 		7		7500-8000	   17
;				 4000-4500 		8		8000-8500	   18
;				 4500-5000		9		8500-9000	   19
; ROUTINES CALLED:
; OUTPUTS:
; COMMENTS:
; SIDE EFFECTS:
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;     26-Feb-2006 Code developed by Gordon Hurford
;     26-Feb-2006 code transformed into a function returning the start frequency of the FST band
;     08-Aug-2006 Added BAND_ID output parameter.

function fst_startfreq,datin,basefreq=basefreq,ifsense=ifsense,band_id=band_id
	lomhz           = datin.lofreq
	bandswitch      = datin.band                                                        ; =0,1 for upper and lower sideband of LO
	sidebandswitch  = datin.sideband                                                    ; =0,1 for 5-9 and 1-5 GHz
	IF bandswitch EQ sidebandswitch THEN ifsense = 1 ELSE ifsense = -1                  ; =1,-1 if RF and IF frequencies have same,different sense
	basefreq        = (10000*(1-bandswitch)*(1-2*sidebandswitch) - lomhz*(1-2*sidebandswitch) -1000) * ifsense  ; =RF frequency for IFfreq=0
	startfreq       =basefreq+500*ifsense                                               ; the lowest RF frecquency in the band
	ifbase = [ 1000, 1500, 2000,-3000, 2500,-3500, 3000,-4000,-4500,-5000,$
	           5000, 5500, 6000, 6500,-7000, 7000,-7500,-8000,-8500,-9000]
	band_id = where(ifbase eq basefreq*ifsense,nbid)
	if (nbid eq 0) then band_id = -1
    return,startfreq
end