;+
; NAME:
;     fst_template
; PURPOSE:
;     To decode a LabView type descriptor and generate a special ACQ template data structure
;     to be used for reading the data files produced by LabView
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;     template=fst_template(unit)
; INPUTS: unit  IDL logical file unit of the file containing the type descriptor
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     lvcode    IDL structure containing the LabView data type codes.
;               If none is provided, the LabView 7.0 codes are used as provided by lvcode70.pro
; ROUTINES CALLED:lvcode70
; OUTPUTS:IDL structure or numeric data template
; COMMENTS:
; SIDE EFFECTS:
; RESTRICTIONS:
;              Only some LabView data types are supported. More will be added as needed
;              This version decode a type descriptor from a file only.
;              The file pointer should point the starting position of the file descriptor
;              In this case, the file should start with a valid LabView type descriptor
;              (The first 4 bytes of the type descriptor should always indicate the length
;               of the type descriptor to be decoded, including the length itself)
;
; MODIFICATION HISTORY:
;     Written 5-May-2005 by Gelu Nita
;     07-March-2006 no longer needed to call fst_hex2dec
;
;-
;--------------------------------------------------------------------------
function fst_template,header,unit,lvcode,unknown=unknown,comment=comment

  ;Header Information is required
 if n_elements(header) eq 0 then message,'Header required'
 if size(header,/type) ne 8 then message,'Header must be a valid IDL structure'

 ;Initialize the unknown datatype counter
 if n_elements(unknown) eq 0 then unknown=0

 t0=systime(/s)

 if n_elements(unit) eq 0 then begin
  openw,unit,'temp.tmp',/get_lun,/SWAP_IF_LITTLE_ENDIAN
  writeu,unit,header.typestr
  free_lun,unit
  openr,unit,'temp.tmp',/get_lun,/SWAP_IF_LITTLE_ENDIAN
  first_call=1
 end

 ;This is the entry pointer. We need it to keep track of the bytes locally read.
 POINT_LUN,-unit,pointer


 ;If no LabView type codes are provided, the LabView 7.0 codes are defined here
 if n_elements(lvcode) eq 0 then lvcode=fst_lvcode70()

 ;The first WORD is the type descriptor length in bytes
 type_length=0u
 readu,unit,type_length

 ;The second WORD is the type code,
 ;but only the least sigificant bit is meaningfull
 ;(the most significant is reserved by LabView for internal purposes !?!?!)
 type_code=0u
 readu,unit,type_code


 CASE (type_code and 255) of

  lvcode.empty:type={EMPTY,CONTAINER:ptr_new()}; This is an empty variant data container

  lvcode.i8:BEGIN
             ;It seems that IDL does not have a signed short integer type
             type=0b
            END

  lvcode.u8:BEGIN
             type=0b
            END

  lvcode.e8:BEGIN
             type=0b
             goto, enumerate
            END

  lvcode.i16:BEGIN
             type=0
            END

  lvcode.u16:BEGIN
             type=0u
            END

  lvcode.e16:BEGIN
             type=0u
             goto, enumerate
            END

  lvcode.i32:BEGIN
             type=0l
            END

  lvcode.e32:BEGIN
             type=0l
             enumerate:
             elements=0u
             readu,unit,elements
             names=strarr(elements)
             total_length=0
             for e=0,elements-1 do begin
              name_length=0b
              readu,unit,name_length
              total_length=total_length+name_length+1
              name=bytarr(name_length)
              readu,unit,name
              names[e]=string(name)
             end
             ;This is needed because it seems that LabView reserves for the name fields an integer number of words.
             SKIP_LUN,unit,total_length mod 2
            END

  lvcode.u32:BEGIN
             type=0ul
            END

  lvcode.single:BEGIN
             type=float(0)
            END

  lvcode.double:BEGIN
             type=0d
            END

  lvcode.string:BEGIN
             ;This is a LONG PASCAL string
             string_length=0L
             readu,unit,string_length
             type=''
            END

  lvcode.variant:BEGIN
             type=ptr_new()
            END

  lvcode.waveform:BEGIN
             ;Waveforms have a subtype for LabView internal use,
             ;we read it just go further.
             subtype_code=0u
             readu,unit,subtype_code
             ;Now we can read the type descriptor of the waveform.
             case subtype_code of
             6: begin
             	;If the subtype_code=6 the "waveform" is in fact a TimeStamp.
             	;Don't ask me why NI defined the TimeStamp (128bit integer)
             	;as a waveform subtype
             	;The ulong type cast is added to fix an apparent bug in the LabView
             	;TimeStamp type stamp definition, which it is claimed to be a sequence of
             	;four positive 32 bit integers, while the returned values in IDL are negative numbers.
             	;I assume here that in fact they are unsigned 32 bit integers.
             	type=ulong(fst_template(unit,lvcode,unknown=unknown))
             	end
             	else:goto,unknown_type
             endcase
            END

  lvcode.cluster: BEGIN
                      ;This is a Cluster data type. It can be
                      ;represented as an IDL structure, if variable names are encoded,
                      ;or as an fixed array of identical unnamed elements
	                  ;Get number of elements in the structure
				      nelements=0u
				      readu,unit,nelements

				      ;Now decode each structure element and add it to the structure
				       for i=0, nelements-1 do begin
				        CASE size(type,/type) of
				         0:type=fst_template(header,unit,lvcode,unknown=unknown)
				         8:type=Create_Struct(type,fst_template(header,unit,lvcode,unknown=unknown))
				         else:type=[type,fst_template(header,unit,lvcode,unknown=unknown)]
				        ENDCASE
				       end
                  END

  lvcode.array:BEGIN
	               ;Read the number of dimensions (word type)
	               dimensions=0u
	               readu,unit,dimensions
	               ;Create a variable to hold the size of each dimensiona and read them
	               ;LabView 7.0 does not store the real sizes in the type descriptor,
	               ;so what we read next are a blind (FFFFFFFF=-1, meaning variable size) for each dimension.
	               ;The real sizes are stored in the data segment imediately preceding the data.
	               arr_sizes=lonarr(dimensions)
	               readu,unit,arr_sizes
	               ;Now read the type descriptor of the array elements
	               element_type=fst_template(header,unit,lvcode,unknown=unknown)
                   type=element_type

               END

  ELSE:BEGIN
           unknown_type:
           POINT_LUN,-unit,count
	       bytes_left=type_length-(count-pointer)
	       unknown=unknown+1
	       descriptor=0b
	       if bytes_left gt 0 then begin
	        descriptor=bytarr(bytes_left)
	        readu,unit,descriptor
	       end
	      type=Create_Struct(strcompress('UNKNOWN'+string(unknown),/rem),{UNKNOWN,Type_Code:to_hex(type_code,4),Type_Descriptor:to_hex(descriptor,2)})
       END
 ENDCASE
   ;There might be some extra bytes unread, that are supposed to hold the variable name, if any
   POINT_LUN,-unit,count
   name=''
   bytes_left=type_length-(count-pointer)
   if bytes_left gt 0 then begin
    ;Read the name length
    name_length=0b
    readu,unit,name_length
    if name_length gt 0 then begin
     name=bytarr(name_length)
     readu,unit,name
     name=string(name)
    ;Just make sure that the file pointer points to the end of the type field
    ;This is needed because it seems that LabView reserves for the name field an integer number of words.
    ;If this is true, we might simply skip one byte if name_length is odd, but it is anyway safer to do it as follows.
     POINT_LUN,-unit,count
     SKIP_LUN,unit,type_length-(count-pointer)
    end
   end
   ;If the variable has a name we return it as a structure, otherwise as an unnamed variable
   if name ne '' then begin
     if n_elements(arr_sizes) gt 0 then begin
      type=Create_Struct(name+'_SIZE',arr_sizes,name,replicate(type,rotate(arr_sizes,2)))
     end else begin
      type=Create_Struct(name,type)
     end
   end

   if keyword_set(first_call) then begin
    free_lun,unit
    if n_tags(type) eq 1 then type=type.(0)
   end

   if keyword_set(comment) then begin
    print,'Data template created in ',systime(/s)-t0, ' seconds'
    print,'Data structure:'
    help, type,/str
   end
   return,type
end