;+
; NAME:
;     fst_variant
; PURPOSE:
;     To read data from a LabView file if the data was saved in a LabView VARIANT format
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;    fst_variant(unit)
; INPUTS:
;        unit:  IDL logical file unit of the file containing the variant data record
;
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;
;        free_unit: set this keyword if you want the file unit to be freed.
;
; ROUTINES CALLED:labview_template, fst_lvdata
; OUTPUTS:IDL structure or numeric data type
; COMMENTS:
; SIDE EFFECTS:
;
; RESTRICTIONS:
;              The logical unit provided must be open with the /SWAP_IF_LITTLE_ENDIAN  keyword set
;              If no logical unit is provided, the user is asked to pick a file that will be automaticale closed when exiting the routine
;              Only some LabView data types are supported. More will be added as needed
;              The file pointer of the logical unit provided should be at the beginning of the variant data section
;
; MODIFICATION HISTORY:
;     Written 25-Jan-2005 by Gelu Nita
;     Rewriten entirely March 7, 2006 GN
;     14-Mar-2006 GN assured that the file units are always closed when explicitely asked for.
;
;-

;--------------------------------------------------------------------------

function fst_variant,type_unit,data_unit,free_unit=free_unit,first_call=first_call,verbose=verbose

    if n_elements(type_unit) eq 0 then begin
      file=dialog_pickfile()
      ;LabView stores data in BIG ENDIAN format
      openr,type_unit,file,/get_lun,/SWAP_IF_LITTLE_ENDIAN
      openr,data_unit,file,/get_lun,/SWAP_IF_LITTLE_ENDIAN
      first_call=1
    end

    if n_elements(data_unit) eq 0 then begin
      status=fstat(type_unit)
      openr,data_unit,status.name,/get_lun,/SWAP_IF_LITTLE_ENDIAN
      first_call=1
    end

    t0=systime(/s)
    ;Each variant record begins with an I32(LONG) record length including the LONG length

    record_length=0L
    READU, type_unit, record_length

    if keyword_set(first_call) then begin
     ;Advance the data_unit pointer
     SKIP_LUN,data_unit,4
    end

    data=fst_lvdata(type_unit,data_unit,first_call=first_call,variable_length=variable_length)


    ;Check for ATTRIBUTES
    n_attributes=0L
    readu,data_unit,n_attributes

    if n_attributes eq 0 then begin
      ;Get rid of the unnecessay structure shell if there is only one tag
      if n_tags(data) eq 1 then data=data.(0)
      ;If the variant container contains no attributes, then return only data
      if keyword_set(free_unit) then begin
       free_lun,type_unit,data_unit
      end
      return,data
    end

    ;READ ATTRIBUTES
    ;The attributes length may be variable, so we need to flag this
     variable_length=1
     for v=1,n_attributes do begin
      key_length=0l
      readu,data_unit,key_length
      key=bytarr(key_length)
      readu,data_unit,key
      key=string(key)
      point_lun,-data_unit,variant
      point_lun,type_unit,variant
      value=fst_variant(type_unit,data_unit,/first_call)
      if n_elements(attributes) eq 0 then attributes=Create_Struct(key,value) else attributes=Create_Struct(attributes,key,value)
     end

    if keyword_set(free_unit) then begin
     free_lun,type_unit,data_unit
    end

    if size(data,/type) eq 8 then begin
     ;If the variant container contains only attributes, then return only them
     if tag_names(data,/structure_name) eq 'EMPTY_VARIANT' then begin
      return,attributes
     end
    end

    ;Make sure that we can concatenate data and attributes
    if n_tags(data) eq 0 then data={data:data}

    result=Create_Struct(data,attributes)

    if keyword_set(verbose) then print,string(N_Tags(data, /data_Length), systime(/s)-t0,$
           format="(i10,' bytes read in',f7.3,' seconds')")

    return,result
end