PRO fst_focisout_reformat, INFILE=infile, OUTFILE=outfile, FOCISOUT=focisout
;
; Reformats focisout file assuming that each input record has the same set of frequencies.
; If this is not the case, an error message is generated and no output file is written.
;
; INFILE  = input filename.  Default = 'focisout.dat'
; OUTFILE = output filename. Default = 'focispect.dat'
; FOCISOUT = named structure to receive output structure (optional).
;
; 17-May-06     Initial version (ghurford@ssl.berkeley.edu)
; 19-May-06 gh  Renamed first properly working version.
;               Change default input to 'focisoutemp.dat'
;               Change default output to 'focisout.dat'
;               Add FOCISOUT keyword.
; 25-May-06 gh  Correct bug in call to fst_focisout_read.
; 24-Jul-06 dg/gh CLOSE and FREE_LUN the input file.
; 28-jUL-06 gh  Add provision for multiband files with uniform sequencing.
;
DEFAULT, infile,    'focisoutemp.dat'
DEFAULT, outfile,   'focispect.dat'
;
; Process the first record to set things up.
input = fst_focisout_read (lunin, /NEWFILE, FILENAME=infile, ACQHEADER=acqheader)
infilestat      = FSTAT(lunin)
acqheader       = input.acqheader
timesectemp     = input.timesec
pwrspectemp     = input.pwrspect                            ; ntime x nfreq x nch array
corspectemp     = input.corspect
nfreq1          = N_ELEMENTS(input.freqmhz)                 ; number of frequencies in one record
ntime1          = N_ELEMENTS(input.timesec)                 ; number of times in one record
nch             = N_ELEMENTS(input.pwrspect)/nfreq1/ntime1
nap             = N_ELEMENTS(input.corspect)/nfreq1/ntime1
freqgroupid     = INTARR(ntime1)                                 ; identifies which frequency group corresponds to input.freqmhz
nfreqgroup      = 1                                         ; number of different frequency groups
freqtab         = REFORM(input.freqmhz, 1, nfreq1)          ; tabulates frequencies for each group in a nfreq x (number of unique frequency groups) array
freqtab[0,*]    = input.freqmhz
nr              = 0                                         ; number of records read so far
eofflag         = EOF(lunin)
;
; Begin loop over subsequent records
WHILE eofflag EQ 0 DO BEGIN
    input           = fst_focisout_read (lunin, ACQHEADER=acqheader)
    nr              = nr+1
;
; Identify the frequency group and check if its a new one.
    freqgroupid     = [freqgroupid, INTARR(ntime1)]
    nt1 = nr * ntime1
    nt2 = nt1 + ntime1 - 1
    newgroup = 1                                            ; flag will be cleared if this frequencies in this record match a previous record
    FOR n = 0, nfreqgroup-1 DO BEGIN
        IF ARRAY_EQUAL(freqtab[n,*], input.freqmhz) EQ 1 THEN BEGIN
            freqgroupid[nt1:nt2] = n
            newgroup = 0
            BREAK                                           ; exit loop once matching group is found
        ENDIF
    ENDFOR
    IF  newgroup NE 0 THEN BEGIN                    ; new set of frequencies
        freqgroupid[nt1:nt2]    = nfreqgroup
        freqtab                 = [freqtab, REFORM(input.freqmhz, 1, nfreq1)]
        nfreqgroup              = nfreqgroup + 1
   ENDIF
;
; Add times and spectra to temporary time-sequenced arrays
    timesectemp    = [timesectemp, input.timesec]
    pwrspectemp    = [pwrspectemp, input.pwrspect]          ; multiple ntime x nfreq x nch array
    corspectemp    = [corspectemp, input.corspect]
    eofflag     = EOF(lunin)                                ; Prepare to exit loop at EOF
ENDWHILE
;
; EOF encountered.
; We now have:  a time-sequenced set of arrays of pwrspect, corspect and timesec;
;               a frequency group ID for every time point;
;               a table (freqtab) that associates each unique frequency group id to a set of frequency values.
;
; TEMPORARY RESTRICTION - ASSUME THE SAME NUMBER OF TIME POINTS IN EACH FREQUENCY GROUP - DISCARD DATA TO MAKE IT SO !!!
;
; Define the output arrays
idhist = HISTOGRAM(freqgroupid)
IF N_ELEMENTS(idhist) NE nfreqgroup THEN MESSAGE, 'Error'
ntime = MIN(idhist)                                         ; number of time points in final output
nfreq = nfreqgroup * nfreq1                                 ; number of frequencies in final output
IF (ntime+1)*nfreq LT nr THEN MESSAGE, 'Sequencing pattern is not supported.'
pwrspect    = FLTARR(ntime, nfreq, nch)
corspect    = COMPLEXARR(ntime, nfreq, nap)
timesec     = DBLARR(ntime)
freqmhz     = FLTARR(nfreq)
timeoff     = FLTARR(nfreq)                      ; time offset (s) of frequency point relative to tabulated timesec
;
; Loop over frequency groups to populate the output arrays.
FOR ng = 0, nfreqgroup-1 DO BEGIN
    ok                      = WHERE(freqgroupid EQ ng)
    nf1                     = ng*nfreq1
    nf2                     = nf1+nfreq1-1
    pwrspect[*,nf1:nf2,*]   = pwrspectemp[ok[0:ntime-1], *,*]
    corspect[*, nf1:nf2,*]  = corspectemp[ok[0:ntime-1],*,*]
    freqmhz[nf1:nf2]        = freqtab[ng,*]
    IF ng EQ 0 THEN BEGIN
        timesec                 = timesectemp[ok[0:ntime-1]]
        timeoff[nf1:nf2]        = 0
    ENDIF ELSE timeoff[nf1:nf2] = MEAN(timesectemp[ok[0:ntime-1]] - timesec)
ENDFOR
;
; Reorder the frequencies.
jf  = SORT(freqmhz)
pwrspect    = pwrspect[*,jf,*]
corspect    = corspect[*,jf,*]
freqmhz     = freqmhz[jf]
timeoff     = timeoff[jf]
;
; Prepare output structure and write it.
focisout = {acqheader: acqheader, $
             timesec: timesec,  $
             freqmhz: freqmhz,  $
            pwrspect: pwrspect, $
            corspect: corspect, $
             timeoff: timeoff }
fst_focisout_write, lunout, focisout, FILENAME=outfile, /HEADER, /NEWFILE
outfilestat = FSTAT(lunout)
CLOSE, lunin
FREE_LUN, lunin
CLOSE, lunout
FREE_LUN, lunout
PRINT, outfile, outfilestat.size/1.E6, FORMAT="('Reformatted results written to  ', A, F13.6, ' Mbytes')"
RETURN
END